/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSGTKMMADDRESSENTRY_H
#define CONEXUSGTKMMADDRESSENTRY_H

#include <set>
#include <conexus.h>


namespace Conexus
{
  namespace Gtk
  {

    /**
      @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
    */
    class AddressContainer
    {
      public:

        typedef std::set<Conexus::IPv4::Address*> IPv4Addresses;
        typedef std::set<Conexus::IPv4::Address*>::iterator IPv4Iterator;
        typedef std::set<Conexus::IPv6::Address*> IPv6Addresses;
        typedef std::set<Conexus::IPv6::Address*>::iterator IPv6Iterator;
        typedef enum ValidateType { IPV4=1, IPV6=2 } ValidateType;

        AddressContainer ( unsigned validate=IPV4|IPV6 );

        virtual ~AddressContainer();

        void connect ( Conexus::IPv4::Address& address );
        void disconnect ( Conexus::IPv4::Address& address );
        void connect ( Conexus::IPv6::Address& address );
        void disconnect ( Conexus::IPv6::Address& address );

        unsigned validate_type();
        void set_validate_type ( unsigned validate );

      protected:
        unsigned m_validate_type;
        IPv4Addresses m_ipv4_address;
        IPv6Addresses m_ipv6_address;

        virtual void on_connect ( Conexus::IPv4::Address& address );
        virtual void on_disconnect ( Conexus::IPv4::Address& address );
        virtual void on_connect ( Conexus::IPv6::Address& address );
        virtual void on_disconnect ( Conexus::IPv6::Address& address );
        virtual void on_validate_type_changed();

    };

  }

}

#endif
