/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "dataprogresswatcher.h"

namespace Conexus
{
  namespace Gtk
  {

    DataProgressWatcher::DataProgressWatcher() : m_data_count ( 0 )
    {
    }


    DataProgressWatcher::~DataProgressWatcher()
    {
    }

    void DataProgressWatcher::add_endpoint ( Conexus::Endpoint::pointer endpoint, bool watch_read, bool watch_write )
    {
      if ( ! endpoint ) return;
      
      if ( m_endpoints.find ( endpoint ) != m_endpoints.end() )
        this->remove_endpoint ( endpoint );

      m_endpoints.insert ( endpoint );
      if ( watch_read )
        m_read_connections[endpoint] =
            endpoint->signal_data_received_size().connect ( sigc::mem_fun ( *this, &DataProgressWatcher::on_read_data_proxy ) );
      if ( watch_write )
        m_write_connections[endpoint] =
            endpoint->signal_data_transmitted_size().connect ( sigc::mem_fun ( *this, &DataProgressWatcher::on_write_data_proxy ) );
    }

    void DataProgressWatcher::remove_endpoint ( Conexus::Endpoint::pointer endpoint )
    {
      if ( ! endpoint ) return;
      
      if ( m_endpoints.find ( endpoint ) == m_endpoints.end() )
        return;

      if ( m_read_connections.find ( endpoint ) != m_read_connections.end() )
      {
        m_read_connections[endpoint].disconnect();
        m_read_connections.erase ( endpoint );
      }

      if ( m_write_connections.find ( endpoint ) != m_write_connections.end() )
      {
        m_write_connections[endpoint].disconnect();
        m_write_connections.erase ( endpoint );
      }

      m_endpoints.erase ( endpoint );
    }

    void DataProgressWatcher::on_read_data_proxy ( size_t n )
    {
      this->on_read_data ( n );
    }

    void DataProgressWatcher::on_write_data_proxy ( size_t n )
    {
      this->on_write_data ( n );
    }

  }

}
