/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "hostentry.h"

namespace Conexus
{
  namespace Gtk
  {

    HostEntry::HostEntry ( unsigned validate ) :
        AddressContainer ( validate ),
        ::Gtk::VBox(),
        m_change_lock ( false )
    {
      this->setup_widget();
    }

    HostEntry::HostEntry ( Conexus::IPv4::Address& address, unsigned validate ) :
        AddressContainer ( validate ),
        ::Gtk::VBox()
    {
      this->setup_widget();
      this->connect ( address );

    }

    HostEntry::HostEntry ( Conexus::IPv6::Address& address, unsigned validate ) :
        AddressContainer ( validate ),
        ::Gtk::VBox()
    {
      this->setup_widget();
      this->connect ( address );
    }

    HostEntry::~HostEntry() {}

    void HostEntry::on_connect ( Conexus::IPv4::Address& address )
    {
      try
      {
        address.set_address ( this->get_hostname() );
      }
      catch ( Conexus::address_exception ) {}
    }

    void HostEntry::on_connect ( Conexus::IPv6::Address& address )
    {
      try
      {
        address.set_address ( this->get_hostname() );
      }
      catch ( Conexus::address_exception ) {}
    }

    void HostEntry::set_hostname ( std::string s )
    {
      m_host.prepend_text ( s );
      m_host_names.insert ( s );
      m_host.set_active_text ( s );
    }

    void HostEntry::on_hostname_changed( )
    {
      std::string hostname;
      AddressContainer::IPv4Iterator v4iter;
      AddressContainer::IPv6Iterator v6iter;

      if ( ( m_validate_type == IPV4 && Conexus::IPv4::is_valid_hostname ( this->get_hostname() ) ) ||
              ( m_validate_type & IPV6 && Conexus::IPv6::is_valid_hostname ( this->get_hostname() ) ) )
      {
        if ( m_image_holder.get_child() != &m_good_host_image )
        {
          m_image_holder.remove();
          m_image_holder.add ( m_good_host_image );
          m_image_holder.show_all_children();
        }
        if ( m_host_names.find ( get_hostname() ) == m_host_names.end() )
        {
          m_host.prepend_text ( get_hostname() );
          m_host_names.insert ( get_hostname() );
        }

        m_hostname_label.set_markup ( this->get_hostname() );
        if ( m_validate_type == IPV4 )
          m_address_label.set_markup ( Conexus::IPv4::address_to_string ( Conexus::IPv4::hostname_to_address ( this->get_hostname() ) ) );
        else
          m_address_label.set_markup ( Conexus::IPv6::address_to_string ( Conexus::IPv6::hostname_to_address ( this->get_hostname() ) ) );

        for ( v4iter = m_ipv4_address.begin(); v4iter != m_ipv4_address.end(); v4iter++ )
          ( *v4iter ) ->set_address ( this->get_hostname() );
        for ( v6iter = m_ipv6_address.begin(); v6iter != m_ipv6_address.end(); v6iter++ )
          ( *v6iter ) ->set_address ( this->get_hostname() );
        m_signal_host_changed.emit ( true );
      }
      else
      {
        if ( m_image_holder.get_child() != &m_bad_host_image )
        {
          m_image_holder.remove();
          m_image_holder.add ( m_bad_host_image );
          m_hostname_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
          m_address_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
          m_signal_host_changed.emit ( false );
        }
      }
    }

    ::Gtk::ComboBoxEntryText& HostEntry::get_hostname_box( )
    {
      return m_host;
    }

    std::string HostEntry::get_hostname( )
    {
      return m_host.get_entry() ->get_text();
    }

    sigc::signal< void, bool > HostEntry::signal_host_changed( )
    {
      return m_signal_host_changed;
    }

    ::Gtk::Image & HostEntry::get_good_hostname_image( )
    {
      return m_good_host_image;
    }

    ::Gtk::Image & HostEntry::get_bad_hostname_image( )
    {
      return m_bad_host_image;
    }

    bool HostEntry::get_show_hostname_label()
    {
      return m_hostname_label.is_visible();
    }

    bool HostEntry::get_show_address_label()
    {
      return m_address_label.is_visible();
    }

    bool HostEntry::get_show_host_image()
    {
      return m_image_holder.is_visible();
    }

    void HostEntry::set_show_hostname_label ( bool show )
    {
      if ( show )
        m_hostname_label.show_all();
      else
        m_hostname_label.hide();
    }

    void HostEntry::set_show_address_label ( bool show )
    {
      if ( show )
        m_address_label.show_all();
      else
        m_address_label.hide();
    }

    void HostEntry::set_show_host_image ( bool show )
    {
      if ( show )
        m_image_holder.show_all();
      else
        m_image_holder.hide();
    }

    void HostEntry::setup_widget( )
    {
      set_homogeneous ( false );

      ::Gtk::HBox* hbox = ::Gtk::manage ( new ::Gtk::HBox() );
      hbox->set_homogeneous ( false );

      hbox->pack_start ( m_host );
      hbox->pack_start ( m_image_holder, ::Gtk::PACK_SHRINK );

      m_host.signal_changed().connect ( sigc::mem_fun ( *this, &HostEntry::on_hostname_changed ) );

      m_good_host_image.set_from_icon_name ( "stock_internet", ::Gtk::IconSize ( ::Gtk::ICON_SIZE_LARGE_TOOLBAR ) );
      m_bad_host_image.set_from_icon_name ( "gtk-dialog-error", ::Gtk::IconSize ( ::Gtk::ICON_SIZE_LARGE_TOOLBAR ) );
      m_image_holder.add ( m_bad_host_image );

      pack_start ( *hbox );

      m_hostname_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
      m_address_label.set_markup ( "<small><i>Invalid Entry</i></small>" );
      m_hostname_label.set_use_markup ( true );
      m_address_label.set_use_markup ( true );

      pack_start ( m_hostname_label );
      pack_start ( m_address_label );

      show_all_children();
    }

    void HostEntry::on_validate_type_changed( )
    {
      on_hostname_changed();
    }

  }

}

