/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSGTKMMHOSTENTRY_H
#define CONEXUSGTKMMHOSTENTRY_H

#include <string>
#include <gtkmm.h>
#include <conexus.h>

#include <conexus-gtkmm/addresscontainer.h>

namespace Conexus
{
  namespace Gtk
  {

    /**
     *
     * TODO provide custom hostname lookups, kinda like DNS but for images...
     * @author Rick L Vinyard Jr
     */
    class HostEntry : public AddressContainer, public ::Gtk::VBox
    {
      public:
        HostEntry ( unsigned validate = IPV4|IPV6 );
        HostEntry ( Conexus::IPv4::Address& address, unsigned validate = IPV4 );
        HostEntry ( Conexus::IPv6::Address& address, unsigned validate = IPV6 );

        ~HostEntry();

        void set_hostname ( std::string s );

        ::Gtk::ComboBoxEntryText& get_hostname_box();
        ::Gtk::Image& get_good_hostname_image();
        ::Gtk::Image& get_bad_hostname_image();

        std::string get_hostname();
        sigc::signal<void, bool> signal_host_changed();

        bool get_show_hostname_label();
        bool get_show_address_label();
        bool get_show_host_image();

        void set_show_hostname_label ( bool show=true );
        void set_show_address_label ( bool show=true );
        void set_show_host_image ( bool show=true );

      protected:
        bool m_change_lock;
        std::set<std::string> m_host_names;
        sigc::connection m_address_connection;

        ::Gtk::Label m_hostname_label;
        ::Gtk::Label m_address_label;
        ::Gtk::ComboBoxEntryText m_host;
        ::Gtk::EventBox m_image_holder;
        ::Gtk::Image m_good_host_image;
        ::Gtk::Image m_bad_host_image;

        sigc::signal<void, bool> m_signal_host_changed;

        virtual void on_hostname_changed();

        virtual void on_connect ( Conexus::IPv4::Address& address );
        virtual void on_connect ( Conexus::IPv6::Address& address );

        virtual void on_validate_type_changed();


      private:
        void setup_widget();

    };

  }

}

#endif
