/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "hostportentry.h"

namespace Conexus
{
  namespace Gtk
  {

    HostPortEntry::HostPortEntry ( Layout layout )
        : ::Gtk::Table(),
        m_layout ( layout )
    {
      m_host_alignment.set ( ::Gtk::ALIGN_LEFT, ::Gtk::ALIGN_TOP );
      m_port_alignment.set ( ::Gtk::ALIGN_LEFT, ::Gtk::ALIGN_TOP );
      m_host_alignment.add ( m_host );
      m_port_alignment.add ( m_port );
      setup_widget();
    }

    HostPortEntry::~HostPortEntry() {}

    HostEntry & HostPortEntry::host_entry( )
    {
      return m_host;
    }

    PortEntry & HostPortEntry::port_entry( )
    {
      return m_port;
    }

    void HostPortEntry::connect ( Conexus::IPv4::Address & address )
    {
      m_host.connect ( address );
      m_port.connect ( address );
    }

    void HostPortEntry::connect ( Conexus::IPv6::Address & address )
    {
      m_host.connect ( address );
      m_port.connect ( address );
    }

    void HostPortEntry::disconnect ( Conexus::IPv4::Address & address )
    {
      m_host.disconnect ( address );
      m_port.disconnect ( address );
    }

    void HostPortEntry::disconnect ( Conexus::IPv6::Address & address )
    {
      m_host.disconnect ( address );
      m_port.disconnect ( address );
    }

    sigc::signal< void, bool > HostPortEntry::signal_host_changed( )
    {
      return m_host.signal_host_changed();
    }

    sigc::signal< void > HostPortEntry::signal_port_changed( )
    {
      return m_port.signal_port_changed();
    }

    std::string HostPortEntry::hostname( )
    {
      return m_host.get_hostname();
    }

    void HostPortEntry::set_hostname ( std::string s )
    {
      m_host.set_hostname ( s );
    }

    uint32_t HostPortEntry::port( )
    {
      return m_port.get_port();
    }

    void HostPortEntry::set_port ( uint32_t p )
    {
      m_port.set_port ( p );
    }

    HostPortEntry::Layout HostPortEntry::layout( )
    {
      return m_layout;
    }

    void HostPortEntry::set_layout ( Layout layout )
    {
      m_layout = layout;
      setup_widget();
    }

    void HostPortEntry::setup_widget( )
    {
      while ( this->get_children().size() > 0 )
        this->remove
        ( ** ( this->get_children().begin() ) );

      switch ( m_layout )
      {
        case VERTICAL:
          attach ( m_host_alignment, 0, 1, 0, 1, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          attach ( m_port_alignment, 0, 1, 1, 2, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          break;
        case HORIZONTAL:
          attach ( m_host_alignment, 0, 1, 0, 1, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          attach ( m_port_alignment, 1, 2, 0, 1, ::Gtk::FILL | ::Gtk::EXPAND, ::Gtk::SHRINK );
          break;
      }
    }

  }

}
