/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSGTKMMHOSTPORTENTRY_H
#define CONEXUSGTKMMHOSTPORTENTRY_H

#include <gtkmm.h>

#include <conexus-gtkmm/hostentry.h>
#include <conexus-gtkmm/portentry.h>

namespace Conexus
{
  namespace Gtk
  {

    /**
    @author Rick L Vinyard Jr
    */
    class HostPortEntry : public ::Gtk::Table
    {
      public:
        typedef enum Layout { VERTICAL, HORIZONTAL } Layout;
        HostPortEntry ( Layout layout = VERTICAL );

        ~HostPortEntry();

        HostEntry& host_entry();
        PortEntry& port_entry();

        std::string hostname();
        void set_hostname ( std::string );

        uint32_t port();
        void set_port ( uint32_t p );

        void connect ( Conexus::IPv4::Address& address );
        void connect ( Conexus::IPv6::Address& address );
        void disconnect ( Conexus::IPv4::Address& address );
        void disconnect ( Conexus::IPv6::Address& address );

        sigc::signal<void, bool> signal_host_changed();
        sigc::signal<void> signal_port_changed();

        Layout layout();
        void set_layout ( Layout layout );

      protected:
        HostEntry m_host;
        PortEntry m_port;
        ::Gtk::Alignment m_host_alignment, m_port_alignment;
        Layout m_layout;

        void setup_widget();
    };

  }

}

#endif
