/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "portentry.h"

#include <sstream>

#define DEFAULT_PORT 1
#define MIN_PORT 0
#define MAX_PORT 32767
#define PORT_INC 1
#define PORT_PAGE_INC 100

namespace Conexus
{
  namespace Gtk
  {

    PortEntry::PortEntry ( unsigned validate )
        : AddressContainer ( validate ),
        ::Gtk::VBox(),
        m_port_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC )
    {
      this->setup_widget();
    }

    PortEntry::PortEntry ( Conexus::IPv4::Address & addr, unsigned validate )
        : AddressContainer ( validate ),
        ::Gtk::VBox(),
        m_port_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC )
    {
      this->setup_widget();
      m_port.set_value ( addr.port() );
      this->connect ( addr );
    }

    PortEntry::PortEntry ( Conexus::IPv6::Address & addr, unsigned validate )
        : AddressContainer ( validate ),
        ::Gtk::VBox(),
        m_port_adjustment ( DEFAULT_PORT, MIN_PORT, MAX_PORT, PORT_INC, PORT_PAGE_INC )
    {
      this->setup_widget();
      m_port.set_value ( addr.port() );
      this->connect ( addr );
    }

    PortEntry::~PortEntry() {}

    ::Gtk::SpinButton & PortEntry::get_port_button( )
    {
      return m_port;
    }

    ::Gtk::Adjustment & PortEntry::get_port_adjustment( )
    {
      return m_port_adjustment;
    }

    void PortEntry::on_connect ( Conexus::IPv4::Address & address )
    {
      address.set_port ( get_port() );
    }

    void PortEntry::on_connect ( Conexus::IPv6::Address & address )
    {
      address.set_port ( get_port() );
    }

    void PortEntry::on_port_changed( )
    {
      uint32_t port = this->get_port();

      if ( m_validate_type == IPV4 )
        m_servicename_label.set_markup ( Conexus::IPv4::service_to_servicename ( port ) );
      else
        m_servicename_label.set_markup ( Conexus::IPv6::service_to_servicename ( port ) );

      for ( IPv4Iterator i = m_ipv4_address.begin(); i != m_ipv4_address.end(); i++ )
        ( *i ) ->set_port ( port );
      for ( IPv6Iterator i = m_ipv6_address.begin(); i != m_ipv6_address.end(); i++ )
        ( *i ) ->set_port ( port );
      m_signal_port_changed.emit();
    }

    void PortEntry::setup_widget( )
    {
      set_homogeneous ( false );

      ::Gtk::HBox* hbox = ::Gtk::manage ( new ::Gtk::HBox() );
      hbox->set_homogeneous ( false );

      hbox->pack_start ( m_port );
      m_port.configure ( m_port_adjustment, 0.5, 0 );

      m_port.signal_changed().connect ( sigc::mem_fun ( *this, &PortEntry::on_port_changed ) );
      m_port.signal_insert_text().connect ( sigc::mem_fun ( *this, &PortEntry::on_spinbutton_text_entry ) );
      m_port.signal_delete_text().connect ( sigc::mem_fun ( *this, &PortEntry::on_spinbutton_text_delete ) );

      pack_start ( *hbox );

      if ( m_validate_type == IPV4 )
        m_servicename_label.set_markup ( Conexus::IPv4::service_to_servicename ( this->get_port() ) );
      else
        m_servicename_label.set_markup ( Conexus::IPv6::service_to_servicename ( this->get_port() ) );

      this->pack_start ( m_servicename_label );
    }

    sigc::signal< void > PortEntry::signal_port_changed( )
    {
      return m_signal_port_changed;
    }

    uint32_t PortEntry::get_port( )
    {
      return m_port.get_value_as_int();
    }

    void PortEntry::set_port ( uint32_t p )
    {
      m_port.set_value ( p );
    }

    void PortEntry::on_spinbutton_text_entry ( const Glib::ustring & text, int * position )
    {
      unsigned port = 0;
      std::stringstream oss;
      oss.str ( m_port.get_text() );
      oss >> port;
      m_port.set_value ( port );
      on_port_changed();
    }

    void PortEntry::on_spinbutton_text_delete ( int start, int end )
    {
      static bool setting = false;
      if ( setting )
        return ;
      setting = true;
      unsigned port = 0;
      std::stringstream oss;
      std::string porttext = m_port.get_text();
      if ( porttext.size() > 0 )
      {
        oss.str ( m_port.get_text() );
        oss >> port;
        m_port.set_value ( port );
        on_port_changed();
      }
      setting = false;
    }

    bool PortEntry::get_show_servicename_label()
    {
      return m_servicename_label.is_visible();
    }

    void PortEntry::set_show_servicename_label ( bool show )
    {
      if ( show )
        m_servicename_label.show_all();
      else
        m_servicename_label.hide();
    }

    void PortEntry::on_validate_type_changed()
    {
      this->on_port_changed();
    }

  }

}
