/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ttybytesizeradio.h"

#include <sstream>

#include <gtkmm/separator.h>

namespace Conexus {
  namespace Gtk {

TTYByteSizeRadio::TTYByteSizeRadio(unsigned byte_size)
 : ::Gtk::Table()
{
  std::ostringstream sout;
  unsigned position;
  ::Gtk::RadioButtonGroup group;
  for (int i=8; i >= 5; i--) {
    sout.str("");
    sout << i;
    position = -i + 8;
    m_radio_button[position].set_label(sout.str());
    this->attach(m_radio_button[ position ], position*2, position*2+1, 0, 1 );
    if ( i != 5 )
    this->attach( *::Gtk::manage( new ::Gtk::VSeparator() ), position*2+1, position*2+2, 0, 1 );
    m_radio_button[position].set_group(group);
    m_radio_button[position].signal_toggled().connect( sigc::bind( sigc::mem_fun( *this, &TTYByteSizeRadio::on_button_changed ), i ) );
  }
  
  this->set_byte_size( byte_size );
}


TTYByteSizeRadio::~TTYByteSizeRadio()
{
}

unsigned TTYByteSizeRadio::byte_size( )
{
  return m_active;
}

void TTYByteSizeRadio::set_byte_size( unsigned byte_size )
{
  if ( byte_size < 5 || byte_size > 8 )
    byte_size = 8;
  m_radio_button[ -byte_size + 8 ].set_active(true);
}

sigc::signal< void > TTYByteSizeRadio::signal_changed( )
{
  return m_signal_changed;
}

void TTYByteSizeRadio::on_button_changed( unsigned byte_size )
{
  m_active = byte_size;
  m_signal_changed.emit();
}

}

}
