/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ttyspeedcombobox.h"

#include <sstream>

namespace Conexus
{
  namespace Gtk
  {

    std::tr1::array<std::string,31> tty_speed_string = { {
        "   0   bps (hang up)",
        "  50   bps",
        "  75   bps",
        " 110   bps",
        " 134   bps",
        " 150   bps",
        " 200   bps",
        " 300   bps",
        " 600   bps",
        "1200   bps",
        "1800   bps",
        "2400   bps",
        "4800   bps",
        "9600   bps",
        "  19.2 Kbps",
        "  38.4 Kbps",
        "  57.6 Kbps",
        " 115.2 Kbps",
        " 230.4 Kbps",
        " 460.8 Kbps",
        " 500.0 Kbps",
        " 576.0 Kbps",
        " 921.6 Kbps",
        "1000   Kbps",
        "1152   Kbps",
        "1500   Kbps",
        "2000   Kbps",
        "2500   Kbps",
        "3000   Kbps",
        "3500   Kbps",
        "4000   Kbps"
      }
    };

    TTYSpeedComboBox::TTYSpeedComboBox ( int active )
        : ::Gtk::ComboBoxText()
    {
      for ( unsigned i = 0; i < tty_speed_string.size(); i++ )
        this->append_text ( tty_speed_string[i] );
      this->set_active ( active );
    }


    TTYSpeedComboBox::~TTYSpeedComboBox()
    {
    }

    unsigned TTYSpeedComboBox::speed( )
    {
      float sp;
      std::istringstream sin ( tty_speed_string[ this->get_active_row_number() ] );
      sin >> sp;
      if ( sp >= 14 )
        sp *= 1000;
      return static_cast<unsigned> ( sp );
    }

    void TTYSpeedComboBox::set_speed ( unsigned speed )
    {
      switch ( speed )
      {
        case 0:
          this->set_active ( 0 );
          break;
        case 50:
          this->set_active ( 1 );
          break;
        case 75:
          this->set_active ( 2 );
          break;
        case 110:
          this->set_active ( 3 );
          break;
        case 134:
          this->set_active ( 4 );
          break;
        case 150:
          this->set_active ( 5 );
          break;
        case 200:
          this->set_active ( 6 );
          break;
        case 300:
          this->set_active ( 7 );
          break;
        case 600:
          this->set_active ( 8 );
          break;
        case 1200:
          this->set_active ( 9 );
          break;
        case 1800:
          this->set_active ( 10 );
          break;
        case 2400:
          this->set_active ( 11 );
          break;
        case 4800:
          this->set_active ( 12 );
          break;
        case 9600:
          this->set_active ( 13 );
          break;
        case 19200:
          this->set_active ( 14 );
          break;
        case 38400:
          this->set_active ( 15 );
          break;
        case 57600:
          this->set_active ( 16 );
          break;
        case 115200:
          this->set_active ( 17 );
          break;
        case 230400:
          this->set_active ( 18 );
          break;
        case 460800:
          this->set_active ( 19 );
          break;
        case 500000:
          this->set_active ( 20 );
          break;
        case 576000:
          this->set_active ( 21 );
          break;
        case 921600:
          this->set_active ( 22 );
          break;
        case 1000000:
          this->set_active ( 23 );
          break;
        case 1152000:
          this->set_active ( 24 );
          break;
        case 1500000:
          this->set_active ( 25 );
          break;
        case 2000000:
          this->set_active ( 26 );
          break;
        case 2500000:
          this->set_active ( 27 );
          break;
        case 3000000:
          this->set_active ( 28 );
          break;
        case 3500000:
          this->set_active ( 29 );
          break;
        case 4000000:
          this->set_active ( 30 );
          break;
        default:
          this->set_active ( 0 );
          break;
      }

    }

  }

}
