/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ttystopbitsradio.h"

#include <sstream>

#include <gtkmm/separator.h>

namespace Conexus
{
  namespace Gtk
  {

    TTYStopBitsRadio::TTYStopBitsRadio ( unsigned stop_bits )
    {
      ::Gtk::RadioButtonGroup group;
      m_stop_bits = stop_bits;

      m_radio_button[0].set_label ( "1" );
      m_radio_button[1].set_label ( "2" );

      this->attach ( m_radio_button[ 0 ], 0, 1, 0, 1 );
      this->attach ( *::Gtk::manage ( new ::Gtk::VSeparator() ), 1, 2, 0, 1 );
      this->attach ( m_radio_button[ 1 ], 2, 3, 0, 1 );

      m_radio_button[0].set_group ( group );
      m_radio_button[1].set_group ( group );

      m_radio_button[0].signal_toggled().connect ( sigc::bind ( sigc::mem_fun ( *this, &TTYStopBitsRadio::on_button_changed ), 1 ) );
      m_radio_button[1].signal_toggled().connect ( sigc::bind ( sigc::mem_fun ( *this, &TTYStopBitsRadio::on_button_changed ), 2 ) );

      this->set_stop_bits ( stop_bits );
    }


    TTYStopBitsRadio::~TTYStopBitsRadio()
    {
    }

    unsigned TTYStopBitsRadio::stop_bits( )
    {
      return m_stop_bits;
    }

    void TTYStopBitsRadio::set_stop_bits ( unsigned stop_bits )
    {
      if ( stop_bits < 1 || stop_bits > 2 )
        stop_bits = 1;
      m_radio_button[ stop_bits-1 ].set_active ( true );
    }

    sigc::signal< void > TTYStopBitsRadio::signal_changed( )
    {
      return m_signal_changed;
    }

    void TTYStopBitsRadio::on_button_changed ( unsigned stop_bits )
    {
      m_stop_bits = stop_bits;
      m_signal_changed.emit();
    }

  }

}
