/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSGTKMMTTYWIDGET_H
#define CONEXUSGTKMMTTYWIDGET_H

#include <gtkmm/table.h>

#include <conexus-gtkmm/serialcombobox.h>
#include <conexus-gtkmm/ttybytesizeradio.h>
#include <conexus-gtkmm/ttyflowcontrolcombobox.h>
#include <conexus-gtkmm/ttyparitycombobox.h>
#include <conexus-gtkmm/ttyparityerrorcombobox.h>
#include <conexus-gtkmm/ttyspeedcombobox.h>
#include <conexus-gtkmm/ttystopbitsradio.h>

namespace Conexus
{
  namespace Gtk
  {

    /**
      @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
    */
    class TTYWidget : public ::Gtk::Table
    {
      public:
        TTYWidget();

        ~TTYWidget();

        void connect ( Conexus::TTY::pointer tty );
        void disconnect( );

        SerialComboBox& device_widget();
        TTYStopBitsRadio& stop_bits_widget();
        TTYByteSizeRadio& byte_size_widget();
        TTYFlowControlComboBox& flow_control_widget();
        TTYParityComboBox& parity_widget();
        TTYParityErrorComboBox& parity_error_widget();
        TTYSpeedComboBox& input_speed_widget();
        TTYSpeedComboBox& output_speed_widget();
        ::Gtk::CheckButton& carrier_detect_widget();
        ::Gtk::CheckButton& receiver_enable_button();

      protected:
        Conexus::TTY::pointer m_tty;

        SerialComboBox m_serial_combobox;
        TTYStopBitsRadio m_stop_bits_radio;
        TTYByteSizeRadio m_byte_size_radio;
        TTYFlowControlComboBox m_flow_control_combobox;
        TTYParityComboBox m_parity_combobox;
        TTYParityErrorComboBox m_parity_error_combobox;
        TTYSpeedComboBox m_input_speed_combobox, m_output_speed_combobox;
        ::Gtk::Button m_link_speed_button;
        bool m_speed_linked;
        Glib::RefPtr<Gdk::Pixbuf> m_link_image, m_unlink_image;
        ::Gtk::CheckButton m_carrier_detect;
        ::Gtk::CheckButton m_receiver_enable_button;

        void on_serial_device_changed();
        void on_byte_size_changed();
        void on_stop_bits_changed();
        void on_flow_control_changed();
        void on_parity_changed();
        void on_parity_error_changed();
        void on_input_speed_changed();
        void on_output_speed_changed();
        void on_speed_link_changed();
        void on_carrier_detect_changed();
        void on_receiver_enable_changed();
    };

  }

}

#endif
