/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSNSPRFILEDESCRIPTOR_H
#define CONEXUSNSPRFILEDESCRIPTOR_H

#include <nspr.h>

#include <conexus/endpoint.h>
#include <conexus-nspr/utility.h>

namespace Conexus
{

  namespace NSPR
  {

    /**
     * @defgroup NSPR Conexus I/O Endpoints with NSPR support
     */

    /**
     * This class is the top level object for all subclasses performing I/O (Input/Output)
     * operations via NSPR file descriptors.
     *
     * @author Rick L Vinyard Jr
     * @ingroup NSPR
     * @ingroup Endpoint
     */
    class FileDescriptor : public Conexus::Endpoint
    {
      protected:

        FileDescriptor( PRFileDesc* fd = NULL, bool close_on_destruct = true );

      public:

        typedef ConexusPointer<FileDescriptor> pointer;

        virtual ~FileDescriptor();

        /** Base class method returning the raw NSPR file descriptor. */
        virtual PRFileDesc* fd() throw ();

        /**
         * Closes the Endpoint object by closing the member file descriptor.
         *
         * If the object is already closed will return without attempting to close.
         */
        virtual void close( bool force = false ) throw ( close_exception );

        virtual size_t input_available() throw();

      protected:
        PRFileDesc* m_fd;

        /**
         * Reimplements write_data() to provide for NSPR specific calls.
         *
         * This method performs blocking I/O whenever timeout is zero or null.
         * This differs from the base Conexus endpoint behavior which performs non-blocking
         * I/O whenever timeout is zero. The reason for this change is due to the fact
         * that NSPR does not support non-blocking I/O at the file level, as only sockets
         * can perform non-blocking I/O in NSPR.
         * 
         * @todo throw specific exceptions rather than a generic exception
         */
        virtual size_t write_data( const Data data, Timeout timeout ) throw ( write_exception );

        /**
         * Implements Endpoint::read_data()
         */
        virtual Data read_data( size_t size, Timeout timeout ) throw ( read_exception );

    };

  }
}

#endif
