/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSNSPRTCP_H
#define CONEXUSNSPRTCP_H

#include <conexus-nspr/socket.h>

namespace Conexus {

  namespace NSPR {

    /**
     * This class encapsulates a socket used for TCP/IP communications.
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup NSPR
     * @ingroup Endpoint
     */
    class TCP : public Socket {
      protected:

        friend class TCPServer;

        TCP( uint32_t address, uint16_t port );

        TCP( const std::string& address, uint16_t port );

      public:
        
        typedef ConexusPointer<TCP> pointer;

        CONEXUS_ENDPOINT_GENERIC_CREATE();

        static pointer create( uint16_t port = 0 );

        static pointer create( uint32_t address, uint16_t port );

        static pointer create( const std::string& address, uint16_t port );

        virtual ~TCP() throw();

        /**
         * Shut down the entirety or part of a TCP connection
         *
         * @param read_write use enum ReadWrite values to indicate which portion of the TCP connection (READ, WRITE or READWRITE) should be shut down
         * @return true if the requested portion of the TCP connection was shut down
         */
        virtual bool shutdown( ReadWrite read_write=READ_WRITE );

        virtual void open() throw (open_exception);

        virtual void connect() throw ( connect_exception );
        virtual void connect( Address& a ) throw ( connect_exception );

      protected:

        /**
         * Protected constructor to be used by TCPServer to create TCP connections.
         * The second bool parameter is used to differentiate the public method from the private method.
         */
        TCP( PRFileDesc* accepted_fd, bool );

        static TCP::pointer create( PRFileDesc* accepted_fd, bool );

        virtual void on_local_address_changed( );
        virtual void on_remote_address_changed( );

        /**
         * Reimplements the Socket ancestor write_data method to ensure that the socket
         * is connected before trying to send.
         */
        virtual size_t write_data( const Data data, Timeout timeout ) throw ( write_exception );

    };

    }

  }

#endif
