/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include <conexus-nspr/tcpserver.h>

#include <conexus/utility.h>

namespace Conexus
{

  namespace NSPR
  {

    TCPServer::TCPServer( uint32_t address, uint16_t port, int backlog ):
        m_backlog(backlog)
    {
      m_listener = TCP::create( );
      m_listener->set_local_address( Conexus::IPv4::Address(address, port) );
    }

    TCPServer::TCPServer( const std::string& address, uint16_t port, int backlog ):
        m_backlog(backlog)
    {
      m_listener = TCP::create( );
      m_listener->set_local_address( Conexus::IPv4::Address(address, port) );
    }

    TCPServer::pointer TCPServer::create( uint16_t port, int backlog )
    {
      return pointer( new TCPServer( INADDR_ANY, port, backlog ) );
    }

    TCPServer::pointer TCPServer::create( uint32_t address, uint16_t port, int backlog )
    {
      return pointer( new TCPServer( address, port, backlog ) );
    }

    TCPServer::pointer TCPServer::create( const std::string& address, uint16_t port, int backlog )
    {
      return pointer( new TCPServer( address, port, backlog ) );
    }

    TCPServer::~TCPServer()
    {}

    Conexus::IPv4::Address& TCPServer::address()
    {
      return m_listener->local_address();
    }

    void TCPServer::set_address( Conexus::IPv4::Address addr )
    {
      m_listener->set_local_address( addr );
    }

    TCP::pointer TCPServer::listener()
    {
      return m_listener;
    }

    const TCP::pointer TCPServer::listener() const
    {
      return m_listener;
    }

    void TCPServer::service_thread_main()
    {
      int n;
      PRIntervalTime interval_time;
      PRFileDesc *fd, *new_connection;
      PRNetAddr addr;
      TCP::pointer newtcp;
      PRPollDesc poll_desc[ 1 ];

      //       struct timeval tv;
      //       fd_set readfd;
      //       int fd;
      //       int new_connection;
      //       struct sockaddr addr;
      //       socklen_t addrlen;
      //       TCP::pointer newtcp;

      fd = m_listener->fd();

      if ( fd == NULL )
        throw -1;

      poll_desc[ 0 ].in_flags = PR_POLL_READ | PR_POLL_EXCEPT;
      poll_desc[ 0 ].fd = fd;

      while ( !m_terminate ) {

        if ( m_timeval.tv_sec == 0 && m_timeval.tv_usec == 0 )
          n = ::PR_Poll( poll_desc, 1, 0 );
        else {
          interval_time = ::PR_SecondsToInterval( m_timeval.tv_sec );
          interval_time += ::PR_MicrosecondsToInterval( m_timeval.tv_usec );
          n = ::PR_Poll( poll_desc, 1, interval_time );
        }

        if ( n > 0 ) {
          new_connection = PR_Accept( fd, &addr, PR_INTERVAL_NO_TIMEOUT );

          if ( new_connection != NULL ) {
            newtcp = TCP::create( new_connection, true );
            this->queue_endpoint( newtcp );
            newtcp.reset(); // Release the smart pointer in the main loop
          }
          // TODO Need to catch exceptions and throw proper errors
          // for now if we have any exceptions or bad i/o we'll stop running
          //           else if ( poll_desc[ 0 ].out_flags & ( PR_POLL_EXCEPT | PR_POLL_ERR | PR_POLL_NVAL ) )
          //             break;
        }


      }

      m_running = false;
    }

    void TCPServer::pre_start( )
    {
      m_listener->bind( );
      m_listener->listen( m_backlog );
    }

    void TCPServer::post_stop()
    {
      m_listener->close();
    }



  } // namespace IPv4

} // namespace Conexus
