/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
/** @cond NSPR */
#ifndef CONEXUSNSPR_TCPSERVER_H
#define CONEXUSNSPR_TCPSERVER_H

#include <conexus/endpointserver.h>
#include <conexus-nspr/tcp.h>

namespace Conexus
{

  namespace NSPR
  {

    /**
     * The TCPServer generates TCP endpoints when a connection request is made.
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup NSPR
     * @ingroup Endpoint
     */
    class TCPServer: public Conexus::EndpointServer
    {
      protected:

        TCPServer( uint32_t address, uint16_t port, int backlog );

        TCPServer( const std::string& address, uint16_t port, int backlog );

      public:

        typedef ConexusPointer<TCPServer> pointer;

        CONEXUS_ENDPOINTSERVER_GENERIC_CREATE();

        static pointer create( uint16_t port = 0, int backlog = 5 );

        static pointer create( uint32_t address, uint16_t port, int backlog = 5 );

        static pointer create( const std::string& address, uint16_t port=0, int backlog = 5 );

        virtual ~TCPServer();

        virtual Conexus::IPv4::Address& address();

        virtual void set_address( Conexus::IPv4::Address addr );

        int backlog();

        TCP::pointer listener();

        const TCP::pointer listener() const;

      protected:
        TCP::pointer m_listener;

        int m_backlog;

        virtual void service_thread_main();

        virtual void pre_start();
        
        virtual void post_stop();

    };

  } // namespace IPv4

} // namespace Conexus

#endif
/** @endcond */
