/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include <conexus-nspr/utility.h>
#include <conexus/utility.h>

#include <termios.h>

namespace Conexus
{
  namespace NSPR
  {

    void init ( bool use_dispatcher_default )
    {
      if ( ! Conexus::initialized() )
        Conexus::init ( use_dispatcher_default );

      if ( ! Conexus::NSPR::initialized() )
        PR_Init ( PR_SYSTEM_THREAD, PR_PRIORITY_NORMAL, 1 );
    }

    bool initialized()
    {
      return PR_Initialized();
    }

    void finalize()
    {
      PR_Cleanup();
    }

    void addr_to_nspr ( const Address& conexusaddr, PRNetAddr& nspraddr )
    {
      struct sockaddr_in* ipv4addr;
      struct sockaddr_in6* ipv6addr;

      switch ( conexusaddr.sockaddr_ptr()->sa_family )
      {
        case AF_INET:
          ipv4addr = ( struct sockaddr_in* ) ( conexusaddr.sockaddr_ptr() );
          nspraddr.inet.family = PR_AF_INET;
          nspraddr.inet.port = ipv4addr->sin_port;
          nspraddr.inet.ip = ipv4addr->sin_addr.s_addr;
          break;
        case AF_INET6:
          ipv6addr = ( struct sockaddr_in6* ) ( conexusaddr.sockaddr_ptr() );
          nspraddr.ipv6.family = PR_AF_INET6;
          nspraddr.ipv6.port = ipv6addr->sin6_port;
          nspraddr.ipv6.flowinfo = ipv6addr->sin6_flowinfo;
          // HACK disabled ipv6
//           for (int i=0; i < 16; i++)
//             nspraddr.ipv6.ip.s6_addr[i] = ipv6addr->sin6_addr.s6_addr[i];
          break;
        default:
          // TODO Throw a proper error
          throw -1;
      }

    }


  }

}
