/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "certificate.h"

#include <pk11func.h>

namespace Conexus
{

  namespace NSS
  {

    Certificate::Certificate ( CERTCertificate* cobj ):
        m_cobj(cobj)
    {
    }

    Certificate::pointer Certificate::create(CERTCertificate * cobj)
    {
      return pointer(new Certificate(cobj) );
    }

    Certificate::pointer Certificate::create(const std::string & nickname, void * wincx)
    {
      CERTCertificate* cobj = PK11_FindCertFromNickname( const_cast<char*>(nickname.c_str()), wincx);

      if ( cobj == NULL ) return pointer();

      return pointer( new Certificate(cobj) );
    }

    Certificate::~Certificate()
    {
      if ( m_cobj ) CERT_DestroyCertificate( m_cobj );
    }

    bool Certificate::verify_name(const std::string & hostname)
    {
      // TODO throw a more proper exception
      if ( m_cobj == NULL ) throw -1;
      
      SECStatus result;
      result = CERT_VerifyCertName( m_cobj, hostname.c_str() );
      return result == SECSuccess;
    }

    CERT_VALIDITY Certificate::check_time_validity(int64_t time, bool allow_override)
    {
      // TODO throw a more proper exception
      if ( m_cobj == NULL ) throw -1;

      SECCertTimeValidity validity;
      validity = CERT_CheckCertValidTimes( m_cobj, time, allow_override);
      return static_cast<CERT_VALIDITY>(validity);
    }

    bool Certificate::ca_names_in_chain(CERTDistNames * ca_names)
    {
      // TODO throw a more proper exception
      if ( m_cobj == NULL ) throw -1;

      SECStatus result;
      result = NSS_CmpCertChainWCANames( m_cobj, ca_names );
      return result == SECSuccess;
    }

    KEA Certificate::kea()
    {
      // TODO throw a more proper exception
      if ( m_cobj == NULL ) throw -1;

      return static_cast<KEA>( NSS_FindCertKEAType( m_cobj ) );
    }

    SSLKEAType Certificate::nss_kea()
    {
      // TODO throw a more proper exception
      if ( m_cobj == NULL ) throw -1;

      return NSS_FindCertKEAType( m_cobj );
    }

    Certificate::operator bool()
    {
      return m_cobj != NULL;
    }

    void Certificate::set_cobj(CERTCertificate * cobj)
    {
      // Makes sure that the current certificate is destroyed only after the
      // new certificate is created in case they are the same certificate
      //
      // This prevents the reference count from dropping to zero before
      // it is incremented in the dup.
      CERTCertificate* cobj2;
      cobj2 = m_cobj;
      m_cobj = NULL;
      if ( cobj != NULL ) m_cobj = CERT_DupCertificate(cobj);
      if ( cobj2 ) CERT_DestroyCertificate( cobj2 );
    }

    CERTCertificate * Certificate::cobj()
    {
      return m_cobj;
    }

  }

}


