/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSNSSCERTIFICATE_H
#define CONEXUSNSSCERTIFICATE_H

#include <string>

#include <cert.h>
#include <certt.h>

#include <conexus/pointer.h>
#include <conexus-nss/enums.h>

namespace Conexus
{

  namespace NSS
  {


    /**
      @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
    */
    class Certificate
    {
      public:
        typedef ConexusPointer<Certificate> pointer;

      protected:
        Certificate(CERTCertificate* cobj=NULL);

      public:

        static pointer create( CERTCertificate* cobj = NULL );

        static pointer create( const std::string& nickname, void* wincx=NULL);

        ~Certificate();

        bool verify_name( const std::string& hostname );

        CERT_VALIDITY check_time_validity( int64_t time, bool allow_override );

        bool ca_names_in_chain( CERTDistNames* ca_names );

        KEA kea();

        SSLKEAType nss_kea();

        operator bool();

        void set_cobj( CERTCertificate* cobj );
        
        CERTCertificate* cobj();

      protected:
        CERTCertificate* m_cobj;
    };

  }

}

#endif
