/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "certificatedb.h"

namespace Conexus
{

  namespace NSS
  {

    CertificateDB::CertificateDB ( CERTCertDBHandle* cobj ) :
        m_cobj ( cobj )
    {
    }

    CertificateDB::pointer CertificateDB::create(CERTCertDBHandle * cobj)
    {
      return pointer( new CertificateDB(cobj) );
    }

    CertificateDB::pointer CertificateDB::create()
    {
      return pointer( new CertificateDB( CERT_GetDefaultCertDB() ) );
    }

    CertificateDB::~CertificateDB()
    {
    }

    bool CertificateDB::verify_certificate_now(Certificate::pointer cert, bool check_sig, CERT_USAGE usage, void * wincx)
    {
      // TODO throw more proper exceptions
      if ( m_cobj == NULL || !cert || cert->cobj() == NULL ) throw -1;

      SECStatus result;
      result = CERT_VerifyCertNow( m_cobj, cert->cobj(), check_sig, static_cast<SECCertUsage>(usage), wincx );
      return result == SECSuccess;
    }

    Certificate::pointer CertificateDB::find_certificate_by_name(SECItem * name)
    {
      // TODO throw more proper exceptions
      if ( m_cobj == NULL ) throw -1;

      return Certificate::create( CERT_FindCertByName( m_cobj, name ) );
    }

    CertificateNicknames::pointer CertificateDB::get_certificate_nicknames(NICKNAMES what, void * wincx)
    {
      // TODO throw more proper exceptions
      if ( m_cobj == NULL ) throw -1;

      return CertificateNicknames::create( CERT_GetCertNicknames( m_cobj, static_cast<int>(what), wincx ) );
    }

    CertificateDB::operator bool()
    {
      return m_cobj != NULL;
    }

    void CertificateDB::set_cobj(CERTCertDBHandle * cobj)
    {
      m_cobj = cobj;
    }

    CERTCertDBHandle * CertificateDB::cobj()
    {
      return m_cobj;
    }

  }

}

