/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSNSSENUMS_H
#define CONEXUSNSSENUMS_H

#include <ssl.h>

namespace Conexus {
  namespace NSS {

    typedef enum EXPORT_POLICY
    {
      EXPORT_NONE,
      US_DOMESTIC,
      US_EXPORTABLE,
      FRANCE,
    } EXPORT_POLICY;

    typedef enum POLICY
    {
      ALLOWED = SSL_ALLOWED,
      RESTRICTED = SSL_RESTRICTED,
      NOT_ALLOWED = SSL_NOT_ALLOWED,
    } POLICY;

    typedef enum CERT_USAGE
    {
      USAGE_SSL_CLIENT = certUsageSSLClient,
      USAGE_SSL_SERVER = certUsageSSLServer,
      USAGE_SSL_SERVER_WITH_STEP_UP = certUsageSSLServerWithStepUp,
      USAGE_SSLCA = certUsageSSLCA,
      USAGE_EMAIL_SIGNER = certUsageEmailSigner,
      USAGE_EMAIL_RECIPIENT = certUsageEmailRecipient,
      USAGE_OBJECT_SIGNER = certUsageObjectSigner,
      USAGE_USER_CERT_IMPORT = certUsageUserCertImport,
      USAGE_VERIFY_CA = certUsageVerifyCA,
      USAGE_PROTECTED_OBJECT_SIGNER = certUsageProtectedObjectSigner,
    } CERT_USAGE;

    typedef enum CERT_VALIDITY {
      VALID = secCertTimeValid,
      EXPIRED = secCertTimeExpired,
      NOT_VALID_YET = secCertTimeNotValidYet,
    } CERT_VALIDITY;

    typedef enum NICKNAMES {
      NICKNAMES_ALL = SEC_CERT_NICKNAMES_ALL,
      NICKNAMES_USER = SEC_CERT_NICKNAMES_USER,
      NICKNAMES_SERVER = SEC_CERT_NICKNAMES_SERVER,
      NICKNAMES_CA = SEC_CERT_NICKNAMES_CA,
    } NICKNAMES;

    typedef enum KEA {
      KEA_NONE = kt_null,
      RSA = kt_rsa,
      DH = kt_dh,
      FORTEZZA = kt_fortezza,
      KEA_SIZE = kt_kea_size,
    } KEA;

    typedef enum SECURITY_STATUS {
      STATUS_OFF = 0,
      STATUS_ON_HIGH = 1,
      STATUS_ON_LOW = 2,
    } SECURITY_STATUS;

  }
}

#endif

