/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "privatekey.h"

#include <key.h>
#include <pk11func.h>

namespace Conexus
{

  namespace NSS
  {

    PrivateKey::PrivateKey(SECKEYPrivateKey* key):
        m_cobj(key)
    {
    }

    PrivateKey::pointer PrivateKey::create(SECKEYPrivateKey * key)
    {
      return pointer( new PrivateKey(key) );
    }

    PrivateKey::pointer PrivateKey::create(Certificate::pointer cert, void * wincx)
    {
      if ( ! cert ) return pointer();
      
      SECKEYPrivateKey* key = PK11_FindKeyByAnyCert( cert->cobj(), wincx );

      if ( key == NULL ) return pointer();

      return pointer( new PrivateKey( key ) );
    }

    PrivateKey::~PrivateKey()
    {
      if ( m_cobj ) SECKEY_DestroyPrivateKey(m_cobj);
    }

    PrivateKey::operator bool()
    {
      return m_cobj != NULL;
    }

    void Conexus::NSS::PrivateKey::set_cobj(SECKEYPrivateKey * cobj)
    {
      m_cobj = cobj;
    }

    SECKEYPrivateKey * Conexus::NSS::PrivateKey::cobj()
    {
      return m_cobj;
    }

  }

}

