/***************************************************************************
 *   Copyright (C) 2008 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ipv4_tcpserver.h"

#include <fcntl.h>

namespace Conexus {

namespace SSL {

namespace IPv4 {

    TCPServer::TCPServer( Context::pointer context, uint32_t address, uint16_t port, int backlog ):
        Conexus::IPv4::TCPServer(address, port, backlog),
        m_context(context)
        {
        }

    TCPServer::TCPServer( Context::pointer context, const std::string& address, uint16_t port, int backlog ):
        Conexus::IPv4::TCPServer( address, port, backlog ),
        m_context(context)
        {
        }

        TCPServer::pointer TCPServer::create( Context::pointer context, uint16_t port, int backlog )
        {
          return pointer( new TCPServer( context, INADDR_ANY, port, backlog ) );
        }

        TCPServer::pointer TCPServer::create( Context::pointer context, uint32_t address, uint16_t port, int backlog )
        {
          return pointer( new TCPServer( context, address, port, backlog ) );
        }

        TCPServer::pointer TCPServer::create( Context::pointer context, const std::string& address, uint16_t port, int backlog )
        {
          return pointer( new TCPServer( context, address, port, backlog ) );
        }

        TCPServer::~TCPServer()
        {
        }

        Context::pointer TCPServer::context()
        {
          return m_context;
        }

        void TCPServer::set_context(Context::pointer c)
        {
          m_context = c;
        }

        void TCPServer::set_blocking()
        {
          int flags;
          if ( ! m_listener->is_open() ) m_listener->open();
          
          flags = ::fcntl(m_listener->fd(), F_GETFL);
          flags &= ~O_NONBLOCK;
          ::fcntl(m_listener->fd(), F_SETFL, flags);
        }

        Conexus::IPv4::TCP::pointer TCPServer::create_accepted_endpoint( int fd )
        {
          TCP::pointer tcp;

          if ( ! m_context ) return TCP::pointer();

          tcp = TCP::create(m_context, fd, true );
          tcp->ssl_accept();
          return tcp;
        }

}

}

}

