/***************************************************************************
 *   Copyright (C) 2008 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUS_SSL_IPV4UDP_H
#define CONEXUS_SSL_IPV4UDP_H

#include <conexus/ipv4_udp.h>
#include <conexus-ssl/ssl.h>

namespace Conexus
{

  namespace SSL
  {

    namespace IPv4
    {

      /**
       * @todo This class is only a preview and is not operational
       *
       * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
       */
      class UDP : public Conexus::IPv4::UDP, public Conexus::SSL::SSL
      {

          UDP( Context::pointer context, uint32_t local_if, uint16_t localport, uint16_t remoteport );

          UDP( Context::pointer context, const std::string& local_if, uint16_t localport, uint16_t remoteport );

        public:
          typedef ConexusPointer<UDP> pointer;

          static pointer create( Context::pointer context, uint16_t localport = 0, uint16_t remoteport = 0 );

          static pointer create( Context::pointer context, uint32_t local_if, uint16_t localport, uint16_t remoteport );

          static pointer create( Context::pointer context, const std::string& local_if, uint16_t localport = 0, uint16_t remoteport = 0 );

          ~UDP() throw ();

          virtual void open() throw ( open_exception );

          virtual void connect() throw ( connect_exception );

          virtual void connect( const Conexus::IPv4::Address& a ) throw ( connect_exception );

        protected:
          
          virtual size_t write_data( const Data data, Timeout timeout ) throw ( write_exception );

          virtual Data read_data( size_t s, Timeout timeout ) throw ( read_exception );

          /**
           * Internally used when an SSL_read indicates that it needs to block on
           * write. For SSL_read we are not concerned with WANT_READ since the
           * block on read is the normal condition when reading.
           */
          bool m_want_write;

          /**
           * Similar to m_want_write, this is used to indicate a write blocked
           * on a WANT_READ. Again, with a write we are not concerned with a
           * WANT_WRITE since that is the normal blocking condition.
           */
          bool m_want_read;

      };

    }

  }

}

#endif
