/***************************************************************************
 *   Copyright (C) 2008 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUS_SSL_IPV6_TCPSERVER_H
#define CONEXUS_SSL_IPV6_TCPSERVER_H

#include <conexus/ipv6_tcpserver.h>

#include <conexus-ssl/context.h>
#include <conexus-ssl/ipv6_tcp.h>

namespace Conexus
{

  namespace SSL
  {

    namespace IPv6
    {

      /**
        @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
      */
      class TCPServer: public Conexus::IPv6::TCPServer
      {
        protected:

          TCPServer ( Context::pointer context, uint32_t address, uint16_t port, int backlog );

          TCPServer ( Context::pointer context, const struct in6_addr& address, uint16_t port, int backlog );

          TCPServer ( Context::pointer context, const std::string& address, uint16_t port, int backlog );

        public:
          typedef ConexusPointer<TCPServer> pointer;

          static pointer create ( Context::pointer context, uint16_t port = 0, int backlog = 5 );

          static pointer create ( Context::pointer context, uint32_t address, uint16_t port, int backlog = 5 );

          static pointer create ( Context::pointer context, const struct in6_addr& address, uint16_t port, int backlog = 5 );

          static pointer create ( Context::pointer context, const std::string& address, uint16_t port=0, int backlog = 5 );

          virtual ~TCPServer();

          Context::pointer context();

          void set_context ( Context::pointer c );

          void set_blocking();

        protected:
          Context::pointer m_context;

          virtual Conexus::IPv6::TCP::pointer create_accepted_endpoint ( int fd );

      };

    }

  }

}

#endif
