/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSADDRESS_H
#define CONEXUSADDRESS_H

#include <netinet/in.h>
#include <netpacket/packet.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <conexus/object.h>

namespace Conexus
{

  /**
   * This top level class provides a pure-virtual inheritance interface for children to inherit from.
   *
   * In concept, this class is much like the BSD raw sockaddr structure.
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class Address: public Object
  {
    public:

      typedef ConexusPointer<Address> pointer;

      Address( );

      Address( const struct sockaddr_storage& addr );

      virtual ~Address();

      /**
       * @return the raw sockaddr structure of this address type
       */
      virtual const struct sockaddr* sockaddr_ptr() const;

      virtual const struct sockaddr_storage* sockaddr_storage_ptr() const;

      /**
       * @return the length of the raw sockaddr structure in bytes
       */
      virtual socklen_t sockaddr_size() const;

      /**
       * @return true if this is considered a broadcast address
       *
       * Is relative to a specific child protocol implementation and returns false unless
       * child provides a specific implementation.
       */
      virtual bool is_broadcast() const;

    protected:

      union {
        struct sockaddr_storage storage;
        struct sockaddr sockaddr;
        struct sockaddr_in ipv4;
        struct sockaddr_in6 ipv6;
        struct sockaddr_ll ll;
      } m_sockaddr;

  };

}

#endif
