/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSENDPOINTSERVER_H
#define CONEXUSENDPOINTSERVER_H

#include <conexus/server.h>
#include <conexus/endpoint.h>

/**
 * A convenience macro to create a static generic_create() method that
 * returns an EndpointServer::pointer rather than the specific type smart pointer.
 *
 * To use this, just put:
 *   CONEXUS_ENDPOINTSERVER_GENERIC_CREATE();
 * somewhere in the public section of your class derived from EndpointServer.
 */
#define CONEXUS_ENDPOINTSERVER_GENERIC_CREATE() static EndpointServer::pointer generic_create() { return create(); }

namespace Conexus
{

  /**
   * The EndpointServer generates Endpoints as a result of operation. This is in
   * contrast to the DataServer which generates data as a result of operation.
   *
   * The canonical example of an EndpointServer is a TCP server which as a result
   * of operation generates accepted TCP connections. Thus, the TCP server is the
   * EndpointServer, while the accepted TCP connections are DataServers.
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   * @ingroup Endpoint
   */
  class EndpointServer : public Server
  {

    protected:

      EndpointServer();

    public:
      typedef ConexusPointer<EndpointServer> pointer;

      virtual ~EndpointServer();

      /**
       * Connect to this signal to receive notifications of new endpoints created by this
       * endpoint server.
       */
      sigc::signal<void, Endpoint::pointer> signal_new_endpoint();

    protected:
      sigc::signal<void, Endpoint::pointer> m_signal_new_endpoint;
      std::queue<Endpoint::pointer> m_endpoint_queue;

      Glib::Mutex m_mutex_signal_endpoint;
      Glib::Mutex m_mutex_endpoint_queue;

      Glib::Dispatcher* m_dispatcher;
      sigc::connection m_dispatcher_connection;

      virtual void queue_endpoint( Endpoint::pointer endpoint );
      void emit( );

  };

}

#endif
