/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSEXCEPT_CLOSE_H
#define CONEXUSEXCEPT_CLOSE_H

#include <conexus/except.h>

namespace Conexus {

/**
 * Base class for all close exceptions.
 * @author Rick L Vinyard Jr
 * @ingroup Exceptions
 */
class close_exception: public conexus_exception {
public:
close_exception(int e=0): conexus_exception("Unknown exception, socket cannot be closed.",e,"close") { }
close_exception(const std::string s, int e=0): conexus_exception(s,e,"close") { }
}
;

namespace exception {


namespace close {

/**
 * EBADF: Thrown when equivalent BSD socket API exception is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup Exceptions
 */
class bad_fd: public close_exception {
public:
  bad_fd(): close_exception(strerror(EBADF), EBADF) { }
}
;

/**
 * EINTR: Thrown when equivalent BSD socket API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup Exceptions
 */
                class interrupted: public close_exception {
public:
  interrupted(): close_exception(strerror(EINTR), EINTR) {}
}
;

/**
 * EIO: Thrown when equivalent BSD socket API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup Exceptions
 */
class io: public close_exception {
public:
  io(): close_exception(strerror(EIO), EIO) { }
}
;

}
}
}
#endif
