/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSEXCEPT_SOCKET_H
#define CONEXUSEXCEPT_SOCKET_H

#include <conexus/except.h>
#include <cstring>

namespace Conexus
  {

    /**
     * Base class for all binding exceptions.
     * @author Rick L Vinyard Jr
     * @ingroup Exceptions
     */
    class socket_exception: public conexus_exception
      {
      public:
      socket_exception(int e=0): conexus_exception(::strerror(e), e,"socket")
        { }
      socket_exception(const std::string s, int e=0): conexus_exception(s, e,"socket")
        { }
      }
    ;

  namespace exception
  {

  namespace socket
      {

      /**
       * EBADF: Thrown when equivalent BSD socket API error is encountered. May be caught alone
       * or via parent class.
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class bad_fd: public socket_exception
        {
        public:
          bad_fd(): socket_exception("sockfd is not a valid descriptor.", EBADF)
          { }
        }
      ;

      /**
       * EFAULT: Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
       * or via parent class.
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class segfault: public socket_exception
      {
        public:
          segfault(): socket_exception("The address pointed to by optval is not in a valid part of the process address space.", EFAULT)
          { }
      }
      ;

      /**
       * EINVAL: Thrown when equivalent BSD socket API error is encountered. May be caught alone
       * or via parent class.
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class optlen_invalid: public socket_exception
        {
        public:
          optlen_invalid(): socket_exception("Socket option optlen is invalid.", EINVAL)
          { }
        }
      ;

      /**
       * ENOPROTOOPT: Thrown when equivalent BSD socket API error is encountered. May be caught alone
       * or via parent class.
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class no_option: public socket_exception
      {
        public:
          no_option(): socket_exception("Socket option is unknown at the level indicated.", ENOPROTOOPT)
          { }
      }
      ;

      /**
       * ENOTSOCK: Thrown when equivalent BSD socket API error is encountered. May be caught alone
       * or via parent class.
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class not_socket: public socket_exception
        {
        public:
          not_socket(): socket_exception("Argument is a descriptor for a file, not a socket.", ENOTSOCK)
          { }
        }
      ;

      }

  }

}

#endif // CONEXUSBIND_ERROR_H
