/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSEXCEPT_STATE_H
#define CONEXUSEXCEPT_STATE_H

#include <conexus/except.h>

namespace Conexus
  {

  /**
   * Base class for all state transition exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup Exceptions
   */
  class state_exception: public conexus_exception
    {
    public:
      state_exception(int e=0): conexus_exception("Unknown exception, state transition failed.",e,"state")
      { }
      state_exception(const std::string s, int e=0): conexus_exception(s,e,"state")
      { }
    }
  ;

  namespace exception
    {

    namespace state
      {

      /**
       * State transition is illegal; i.e. not supported state, such as UNCHANGED
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class illegal: public state_exception
        {
        public:
          illegal(): state_exception("Attempting to transition to an illegal state.", -1)
          { }
        }
      ;

      /**
       * State transition is inconsistent such as OPENED|CLOSED
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class inconsistent: public state_exception
      {
      public:
      inconsistent(): state_exception("Attempting to transition to an inconsistent state.", -1)
        { }
      }
      ;

      /**
       * The state transition failed. The state transition was a legal state, the request
       * was not inconsistent, but for one reason or another it still failed.
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class failed: public state_exception
      {
      public:
      failed(): state_exception("State transition was legal and consistent, but it still failed.", -1)
        { }
      };

      /**
       * Invalid read/write mode
       * @author Rick L Vinyard Jr
       * @ingroup Exceptions
       */
      class invalid_read_write: public state_exception
      {
        public:
          invalid_read_write(): state_exception("Invalid read/write mode", -1)
          { }
      };


      }

  }

}

#endif // CONEXUSSEND_ERROR_H
