/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSFILEDESCRIPTOR_H
#define CONEXUSFILEDESCRIPTOR_H

#include <conexus/endpoint.h>

namespace Conexus
{

  /**
   * This class is the top level object for all subclasses performing I/O (Input/Output)
   * operations via file descriptors (which in Linux/Unix/POSIX can be used for almost all
   * I/O).
   *
   * @todo think about whether read_data() should throw select_interrupted() by default
   * 
   * @author Rick L Vinyard Jr
   * @ingroup Endpoint
   */
  class FileDescriptor : public Endpoint
  {
    protected:

      FileDescriptor( int fd = -1, bool close_on_destruct = true );

    public:

      typedef ConexusPointer<FileDescriptor> pointer;

      virtual ~FileDescriptor();

      /** Base class method returning the raw file descriptor. */
      virtual int fd() throw ();

      /**
       * Closes the Endpoint object by closing the member file descriptor.
       *
       * If the object is already closed will return without attempting to close.
       */
      virtual void close( bool force = false ) throw ( close_exception );

      bool is_blocking();

      bool set_blocking( bool block=true );

    protected:
      int m_fd;

      virtual size_t write_data( const Data data, Timeout to ) throw ( write_exception );

      virtual Data read_data( size_t s, Timeout to ) throw ( read_exception );

  };

}

#endif
