/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSIP_H
#define CONEXUSIP_H

#include <conexus/socket.h>
#include <conexus/ipaddress.h>

namespace Conexus
{

  /**
   * This class serves as the base class for all IP communication classes.
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   * @ingroup Endpoint
   */
  class IP : public Socket
  {

    protected:

      IP( int domain = -1, int type = -1, int protocol = 0 );

    public:
      /** Class typedef to smart pointer */
      typedef ConexusPointer<IP> pointer;

      virtual ~IP() throw();

      virtual IPAddress& generic_local_address() = 0;

      virtual IPAddress& generic_remote_address() = 0;

      int ttl();

      void set_ttl( int t );

      TOS tos();

      void set_tos( TOS t );

  };

}

#endif
