/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ipv4_ip.h"

namespace Conexus
  {
  namespace IPv4
    {

    IP::IP(int type, int protocol)
      : Conexus::IP(PF_INET, type, protocol),
        m_remote_address_set(false)
    {
      m_local_address.signal_changed().connect(sigc::mem_fun(*this, &IP::on_local_address_changed_proxy));
      m_remote_address.signal_changed().connect(sigc::mem_fun(*this, &IP::on_remote_address_changed_proxy));
    }


    IP::~IP() throw ()
    {}

    void IP::bind() throw (bind_exception)
    {
      Socket::bind(m_local_address);
    }

    void IP::bind(const Conexus::Address& a) throw (bind_exception) {
      m_local_address = a;
      Socket::bind(a);
    }

    void IP::connect() throw ( connect_exception ) {
      if ( m_remote_address_set )
        Socket::connect( m_remote_address );
      else
        throw exception::connect::no_remote_address();
    }

    void IP::connect( const Address& a ) throw ( connect_exception ) {
      m_remote_address = a;
      Socket::connect( m_remote_address );
    }

    IPAddress& IP::generic_local_address() {
      return m_local_address;
    }

    IPAddress& IP::generic_remote_address() {
      return m_remote_address;
    }

    Address & IP::local_address( )
    {
      return m_local_address;
    }

    Address & IP::remote_address( )
    {
      return m_remote_address;
    }

    void IP::set_remote_address( const Address& addr )
    {
      m_remote_address = addr;
      m_remote_address_set = true;
    }

    void IP::set_remote_address( uint32_t addr, uint16_t port )
    {
      m_remote_address = Address(addr, port);
      m_remote_address_set = true;
    }

    void IP::set_remote_address( const std::string& addr, uint16_t port )
    {
      m_remote_address = Address(addr, port);
      m_remote_address_set = true;
    }

    void IP::unset_remote_address()
    {
      m_remote_address = Address();
      m_remote_address_set = false;
    }

    void IP::set_local_address( const Address& addr )
    {
      m_local_address = addr;
    }

    void IP::set_local_address( uint32_t addr, uint16_t port )
    {
      m_local_address = Address(addr, port);
    }

    void IP::set_local_address( const std::string& addr, uint16_t port )
    {
      m_local_address = Address(addr, port);
    }

    void IP::on_local_address_changed() {
    }

    void IP::on_remote_address_changed() {
    }

    void IP::on_local_address_changed_proxy() {
      this->on_local_address_changed( );
    }

    void IP::on_remote_address_changed_proxy() {
      this->on_remote_address_changed( );
      m_remote_address_set = true;
    }

    }
}
