/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ipv6_tcpserver.h"

#include "ipv6_tcp.h"

#include <conexus/utility.h>

namespace Conexus
{

  namespace IPv6
  {

    TCPServer::TCPServer( uint32_t address, uint16_t port, int backlog ):
        m_backlog(backlog)
    {
      m_listener = TCP::create( );
      m_listener->set_local_address( address, port );
    }

    TCPServer::TCPServer( const struct in6_addr& address, uint16_t port, int backlog ):
        m_backlog(backlog)
    {
      m_listener = TCP::create( );
      m_listener->set_local_address( address, port );
    }

    TCPServer::TCPServer( const std::string& address, uint16_t port, int backlog ):
        m_backlog(backlog)
    {
      m_listener = TCP::create( );
      m_listener->set_local_address( address, port );
    }

    TCPServer::pointer TCPServer::create( uint16_t port, int backlog )
    {
      return pointer( new TCPServer( in6addr_any, port, backlog ) );
    }

    TCPServer::pointer TCPServer::create( uint32_t address, uint16_t port, int backlog )
    {
      return pointer( new TCPServer( address, port, backlog ) );
    }

    TCPServer::pointer TCPServer::create( const struct in6_addr& address, uint16_t port, int backlog )
    {
      return pointer( new TCPServer( address, port, backlog ) );
    }

    TCPServer::pointer TCPServer::create( const std::string& address, uint16_t port, int backlog )
    {
      return pointer( new TCPServer( address, port, backlog ) );
    }

    TCPServer::~TCPServer()
    {}

    Address& TCPServer::address()
    {
      return m_listener->local_address();
    }

    void TCPServer::set_address( Address addr )
    {
      m_listener->set_local_address( addr );
    }

    int TCPServer::backlog()
    {
      return m_backlog;
    }

    TCP::pointer TCPServer::listener()
    {
      return m_listener;
    }

    const TCP::pointer TCPServer::listener() const
    {
      return m_listener;
    }

    void TCPServer::set_reuse_address( bool set )
    {
      m_listener->set_reuse_address(set);
    }

    void TCPServer::service_thread_main()
    {
      int n;
      struct timeval tv;
      fd_set readfd;
      int fd;
      int new_connection;
      struct sockaddr addr;
      socklen_t addrlen;
      TCP::pointer newtcp;

      fd = m_listener->fd();

      if ( fd < 0 )
        throw;

      while ( !m_terminate ) {
        FD_ZERO( &readfd );
        FD_SET( fd, &readfd );

        if ( m_timeval.tv_usec <= 0 )
          n = select( fd + 1, &readfd, NULL, NULL, NULL );
        else {
          tv = m_timeval;
          n = select( fd + 1, &readfd, NULL, NULL, &tv );
        }
        if ( FD_ISSET( fd, &readfd ) ) {
          new_connection = ::accept( fd, &addr, &addrlen );
          newtcp = this->create_accepted_endpoint( new_connection );
          this->queue_endpoint( newtcp );
        }
      }
      m_running = false;
    }

    void TCPServer::pre_start( )
    {
      m_listener->bind( );
      m_listener->listen( m_backlog );
    }

    void TCPServer::post_stop()
    {
      m_listener->close();
    }

    TCP::pointer TCPServer::create_accepted_endpoint(int fd)
    {
      return TCP::create( fd, true );
    }

  } // namespace IPv6

} // namespace Conexus
