/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef IPV6IPV6_TCPSERVER_H
#define IPV6IPV6_TCPSERVER_H

#include <conexus/ipv6_address.h>
#include <conexus/endpointserver.h>
#include <conexus/ipv6_tcp.h>

/**
 * \example ipv6_tcp_server.cpp
 * Conexus::IPv6::TCPServer Example
 *
 * \example ipv6_tcp_echo_server.cpp
 * Conexus::IPv6::TCPServer Echo Server Example
 */

namespace Conexus
{

  namespace IPv6
  {

    /**
     * The TCPServer generates TCP endpoints when a connection request is made.
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup IPv6
     * @ingroup Endpoint
     */
    class TCPServer: public Conexus::EndpointServer
    {

      protected:

        TCPServer( uint32_t address, uint16_t port, int backlog=5 );

        TCPServer( const struct in6_addr& address, uint16_t port, int backlog=5 );

        TCPServer( const std::string& address, uint16_t port, int backlog=5 );

      public:

        typedef ConexusPointer<TCPServer> pointer;

        CONEXUS_ENDPOINTSERVER_GENERIC_CREATE();

        static TCPServer::pointer create( uint16_t port = 0, int backlog=5 );

        static TCPServer::pointer create( uint32_t address, uint16_t port, int backlog=5 );

        static TCPServer::pointer create( const struct in6_addr& address, uint16_t port = 0, int backlog=5 );

        static TCPServer::pointer create( const std::string& address, uint16_t port = 0, int backlog=5 );

        virtual ~TCPServer();

        virtual Address& address();

        virtual void set_address( Address addr );

        int backlog();

        TCP::pointer listener();

        const TCP::pointer listener() const;

        void set_reuse_address( bool set=true );

      protected:
        TCP::pointer m_listener;
        int m_backlog;

        virtual void pre_start();
        
        virtual void post_stop();

        virtual void service_thread_main();

        virtual TCP::pointer create_accepted_endpoint( int fd );

    };

  } // namespace IPv6

} // namespace Conexus

#endif
