/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "ll_address.h"

#include <netinet/in.h>
#include <sys/ioctl.h>
#include <net/if.h>

namespace Conexus
{
  namespace LL
  {

    Address::Address( unsigned short protocol )
    {
      m_sockaddr.ll.sll_family = AF_PACKET;
      m_sockaddr.ll.sll_protocol = htons( protocol );
    }

    Address::~Address()
    {}

    unsigned short Address::family() const
    {
      return m_sockaddr.ll.sll_family;
    }

    unsigned short Address::protocol() const
    {
      return ntohs( m_sockaddr.ll.sll_protocol );
    }

    void Address::set_protocol( unsigned short p )
    {
      m_sockaddr.ll.sll_protocol = htons( p );
    }

    int Address::interface_index() const
    {
      return m_sockaddr.ll.sll_ifindex;
    }

    unsigned short Address::hardware_type() const
    {
      return m_sockaddr.ll.sll_hatype;
    }

    unsigned char Address::packet_type() const
    {
      return m_sockaddr.ll.sll_pkttype;
    }

    unsigned char Address::hardware_address_length() const
    {
      return m_sockaddr.ll.sll_halen;
    }

    const unsigned char* Address::hardware_address() const
    {
      return m_sockaddr.ll.sll_addr;
    }

    socklen_t Address::sockaddr_size() const
    {
      return sizeof( struct sockaddr_ll );
    }

    bool Address::is_broadcast() const
    {
      return ( m_sockaddr.ll.sll_pkttype == PACKET_BROADCAST );
    }

    void Address::set_interface( int ifindex )
    {
      set_interface( Conexus::interface_name( ifindex ) );
    }

    void Address::set_interface( const std::string ifname )
    {
      int sd = socket( PF_PACKET, SOCK_RAW, 0 );
      struct ifreq req;
      strncpy( req.ifr_name, ifname.c_str(), IFNAMSIZ );
      if ( ioctl( sd, SIOCGIFINDEX, &req ) >= 0 ) {
        m_sockaddr.ll.sll_ifindex = req.ifr_ifindex;
        if ( ioctl( sd, SIOCGIFHWADDR, &req ) >= 0 ) {
          m_sockaddr.ll.sll_hatype = req.ifr_hwaddr.sa_family;
          m_sockaddr.ll.sll_halen = 8;
          memcpy( m_sockaddr.ll.sll_addr, req.ifr_hwaddr.sa_data, 8 );
        }
      }
      close( sd );
    }

  }
}

