/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUS_LLLL_ADDRESS_H
#define CONEXUS_LLLL_ADDRESS_H

#include <net/ethernet.h>

#include <vector>
#include <string>

#include <conexus/address.h>
#include <conexus/utility.h>

namespace Conexus
  {

  /**
   * @namespace Conexus::LL
   * The conexus::LL (Link Layer) namespace contains classes capable of communicating directly at the
   * OSI Layer 2 and allows protocol modules to be implemented in user space directly
   * on top of the physical layer [ For more information see PACKET(7) ]
   */
  namespace LL
    {

    /**
     * Link Layer packet interface device address.
     *
     * This object wraps the Linux device level packet interface structure sockaddr_ll
     * and provides accessor methods for the members. Many of the related
     * functions for manipulating the sockaddr_ll structure are presented in the class
     * interface as methods.
     *
     * Several extensions to the API functionality are also provided, such as a method to
     * retrieve the kernel's numeric index for a give interface or the MAC address.
     *
     * @author Rick L Vinyard Jr
     * @ingroup conexus
     */
    class Address : public Conexus::Address
      {
      public:
        Address(unsigned short protocol=ETH_P_IP);

        virtual ~Address();

        unsigned short family() const;

        unsigned short protocol() const;
        void set_protocol(unsigned short);

        int interface_index() const;
        unsigned short hardware_type() const;
        unsigned char packet_type() const;
        unsigned char hardware_address_length() const;
        const unsigned char* hardware_address() const;

        virtual socklen_t sockaddr_size() const;
        virtual bool is_broadcast() const;

        void set_interface(int);
        void set_interface(const std::string);

      protected:

        friend class Packet;

      };

  }

}

#endif
