/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSLL_PACKET_H
#define CONEXUSLL_PACKET_H

#include <conexus/socket.h>
#include <conexus/ll_address.h>

/**
 * \example ll_client.cpp
 * Conexus::LL::Packet link layer client example
 *
 * \example ll_server.cpp
 * Conexus::LL::Packet link layer server example
 */


namespace Conexus
{

  namespace LL
  {

    /**
     * A link-layer packet endpoint.
     * 
     * @author Rick L Vinyard Jr
     * @ingroup conexus
     * @ingroup Endpoint
     */
    class Packet : public Socket
    {
      protected:

        Packet( int protocol = ETH_P_IP, int type = SOCK_DGRAM );

      public:

        typedef ConexusPointer<Packet> pointer;

        CONEXUS_ENDPOINT_GENERIC_CREATE();

        static pointer create( int protocol = ETH_P_IP, int type = SOCK_DGRAM );

        virtual ~Packet() throw();

        virtual void bind() throw ( bind_exception );

        virtual void bind( Address& a ) throw ( bind_exception );

        void set_interface( int );

        void set_interface( const std::string );

        int interface_index( const std::string name );

        std::string interface_name( int ifindex );

        std::vector<std::string> interface_names();

        virtual size_t input_available() throw();

      protected:
        Address m_address;
        int m_protocol;

        virtual size_t write_data( const Data data, Timeout timeout ) throw ( write_exception );

    };

  }

}

#endif
