/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSMESSAGEQUEUE_H
#define CONEXUSMESSAGEQUEUE_H

#include <mqueue.h>
#include <sys/stat.h>

#include <conexus/endpoint.h>

/**
 * \example messagequeue_client.cpp
 * Conexus::MessageQueue example client
 *
 * \example messagequeue_server.cpp
 * Conexus::MessageQueue example server
 */

namespace Conexus
{

  /**
   * This class encapsulates a POSIX message queue.
   *
   * \todo Drive message queues off notifications rather than the timeout mechanism of Endpoint
   *
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   * @ingroup Endpoint
   */
  class MessageQueue : public Endpoint
  {
    protected:

      MessageQueue( const std::string& name,
                    unsigned flags,
                    mode_t mode,
                    bool close_on_destruct
                  );

    public:

      typedef ConexusPointer<MessageQueue> pointer;

      CONEXUS_ENDPOINT_GENERIC_CREATE();

      static MessageQueue::pointer create( const std::string& name=std::string(),
                                           unsigned flags=QUEUE_READ|QUEUE_WRITE|QUEUE_CREATE,
                                           mode_t mode = S_IRWXU,
                                           bool close_on_destruct=true
                                         );

      virtual ~MessageQueue();

      virtual void open() throw ( open_exception );

      virtual void close( bool force = false ) throw ( close_exception );

      const std::string& name() const;

      void set_name( const std::string& name );

      unsigned flags() const;

      void set_flags(unsigned flags);

      bool unlink_on_destruction();

      void set_unlink_on_destruction( bool u=true );

      long int max_messages();

      long int max_message_size();

    protected:
      std::string m_name;
      unsigned m_flags;
      bool m_unlink_on_destruction;
      long int m_max_messages;
      long int m_max_message_size;

      mode_t m_mode;

      mqd_t m_queue;

      virtual size_t write_data( const Data data, Timeout timeout ) throw ( write_exception );

      virtual Data read_data( size_t size, Timeout timeout ) throw ( read_exception );
  };

}

#endif
