/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSPIPE_H
#define CONEXUSPIPE_H

#include <conexus/filedescriptor.h>

/**
 * \example anonymous_pipe.cpp
 * Conexus::Pipe anonymous pipe example
 *
 * \example anonymous_pipe_server.cpp
 * Conexus::Pipe anonymous pipe server example
 */

namespace Conexus
{

  /**
   * This class encapsulates a POSIX pipe.
   *
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   * @ingroup Endpoint
   */
  class Pipe : public FileDescriptor
  {
    protected:

      Pipe( );

    public:

      typedef ConexusPointer<Pipe> pointer;

      CONEXUS_ENDPOINT_GENERIC_CREATE();

      static Pipe::pointer create( );

      virtual ~Pipe();

      virtual void open() throw ( open_exception );

      virtual void close( bool force = false ) throw ( close_exception );

      virtual void close_read( bool force = false ) throw ( close_exception );

      virtual void close_write( bool force = false ) throw ( close_exception );

      virtual size_t input_available() throw();

    protected:
      int m_write_fd;

      virtual size_t write_data( const Data data, Timeout timeout ) throw ( write_exception );

      /**
       * Reimplements Endpoint::read_data() using FileDescriptor::read_data().
       *
       * Changes from FileDescriptor::read_data()
       *   - If read size is zero, throws Conexus::exception::read::disconnected
       */
      virtual Data read_data( size_t size, Timeout timeout ) throw ( read_exception );
  };

}

#endif
