/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "registry.h"

namespace Conexus
{

  Registry::Registry()
  {}


  Registry::pointer Registry::create( )
  {
    return pointer ( new Registry() );
  }

  Registry::~Registry()
  {
  }

  bool Registry::add ( const std::string & name, Endpoint::pointer obj )
  {
    // The name must be unique for all objects in this registry
    if ( m_name_endpoint.find ( name ) != m_name_endpoint.end() || m_name_endpointserver.find ( name ) != m_name_endpointserver.end() )
      return false;

    m_name_endpoint[name] = obj;

    m_signal_added.emit(name);

    return true;
  }

  bool Registry::add ( const std::string & name, EndpointServer::pointer obj )
  {
    // The name must be unique for all objects in this registry
    if ( m_name_endpoint.find ( name ) != m_name_endpoint.end() || m_name_endpointserver.find ( name ) != m_name_endpointserver.end() )
      return false;

    m_name_endpointserver[name] = obj;

    m_signal_added.emit(name);

    return true;
  }

  bool Registry::remove ( const std::string & name )
  {
    NameEndpointMap::iterator name_ep_iter;
    NameEndpointServerMap::iterator name_eps_iter;

    name_ep_iter = m_name_endpoint.find ( name );
    if ( name_ep_iter != m_name_endpoint.end() )
    {
      m_name_endpoint.erase ( name_ep_iter );
      m_signal_removed.emit(name);
      return true;
    }

    name_eps_iter = m_name_endpointserver.find ( name );
    if ( name_eps_iter != m_name_endpointserver.end() )
    {
      m_name_endpointserver.erase ( name_eps_iter );
      m_signal_removed.emit(name);
      return true;
    }

    return false;
  }

  bool Registry::remove ( Endpoint::pointer obj )
  {
    NameEndpointMap::iterator iter;

    for ( iter = m_name_endpoint.begin(); iter != m_name_endpoint.end(); iter++ )
    {
      if ( iter->second == obj )
      {
        std::string name = iter->first;
        m_name_endpoint.erase ( iter );
        m_signal_removed.emit(name);
        return true;
      }
    }
    return false;
  }

  bool Registry::remove ( EndpointServer::pointer obj )
  {
    NameEndpointServerMap::iterator iter;

    for ( iter = m_name_endpointserver.begin(); iter != m_name_endpointserver.end(); iter++ )
    {
      if ( iter->second == obj )
      {
        std::string name = iter->first;
        m_name_endpointserver.erase ( iter );
        m_signal_removed.emit(name);
        return true;
      }
    }
    return false;
  }

  Object::pointer Registry::lookup ( const std::string & name )
  {
    NameEndpointMap::iterator ep_iter = m_name_endpoint.find( name );
    if ( ep_iter != m_name_endpoint.end() )
      return ep_iter->second;

    NameEndpointServerMap::iterator eps_iter = m_name_endpointserver.find( name );
    if ( eps_iter != m_name_endpointserver.end() )
      return eps_iter->second;

    return Object::pointer();
  }

  Endpoint::pointer Registry::lookup_endpoint ( const std::string & name )
  {
    NameEndpointMap::iterator ep_iter = m_name_endpoint.find( name );
    if ( ep_iter != m_name_endpoint.end() )
      return ep_iter->second;
    return Endpoint::pointer();
  }

  EndpointServer::pointer Registry::lookup_server ( const std::string & name )
  {
    NameEndpointServerMap::iterator eps_iter = m_name_endpointserver.find( name );
    if ( eps_iter != m_name_endpointserver.end() )
      return eps_iter->second;
    return EndpointServer::pointer();
  }

  const Registry::NameEndpointMap& Registry::endpoints( )
  {
    return m_name_endpoint;
  }

  const Registry::NameEndpointServerMap& Registry::servers( )
  {
    return m_name_endpointserver;
  }

  sigc::signal< void, const std::string & > Registry::signal_added( )
  {
    return m_signal_added;
  }

  sigc::signal< void, const std::string & > Registry::signal_removed( )
  {
    return m_signal_removed;
  }

}



