/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSREGISTRY_H
#define CONEXUSREGISTRY_H

#include <map>

#include <conexus/pointer.h>
#include <conexus/endpoint.h>
#include <conexus/endpointserver.h>

/**
 * \example registry_client.cpp
 * Conexus::Registry client example
 *
 * \example registry_server.cpp
 * Conexus::Registry server example
 */

namespace Conexus {

/**
 * This class serves as a registy for named endpoints and endpoint servers.
 *
 * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
 */
class Registry{
public:

  typedef ConexusPointer<Registry> pointer;

  typedef std::map<std::string, Endpoint::pointer> NameEndpointMap;

  typedef std::map<std::string, EndpointServer::pointer> NameEndpointServerMap;

    Registry();

    static pointer create();

    ~Registry();

    /**
     * Adds the endpoint obj to the registry.
     * The name must be unique for this registry.
     *
     * @return true if obj is successfully added, false otherwise
     */
    bool add( const std::string& name, Endpoint::pointer obj );

    /**
     * Adds the endpoint server obj to the registry.
     * The name must be unique for this registry.
     *
     * @return true if obj is successfully added, false otherwise
     */
    bool add( const std::string& name, EndpointServer::pointer obj );

    /** Removed the named obj from the registry */
    bool remove( const std::string& name );

    bool remove( Endpoint::pointer obj );

    bool remove( EndpointServer::pointer obj );

    Object::pointer lookup( const std::string& name );

    Endpoint::pointer lookup_endpoint( const std::string& name );

    EndpointServer::pointer lookup_server( const std::string& name );

    const NameEndpointMap& endpoints();

    const NameEndpointServerMap& servers();

    sigc::signal<void,const std::string&> signal_added();

    sigc::signal<void,const std::string&> signal_removed();

  protected:

    NameEndpointMap m_name_endpoint;

    NameEndpointServerMap m_name_endpointserver;

    sigc::signal<void,const std::string&> m_signal_added;

    sigc::signal<void,const std::string&> m_signal_removed;

};

}

#endif
