/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "server.h"

#include <sys/select.h>

#include <conexus/utility.h>

namespace Conexus
{

  Server::Server() :
      m_thread( NULL ),
      m_running( false ),
      m_dispatcher( NULL ),
      m_use_dispatcher(false)
  {
    m_timeval.tv_sec = 0;
    m_timeval.tv_usec = 500000;
  }


  Server::~Server()
  {
    if ( m_running )
      stop();
  }

  void Server::start( )
  {
    this->start( use_dispatcher_default() );
  }

  void Server::start( bool use_dispatcher )
  {
    if ( m_running ) return ;

    m_use_dispatcher = use_dispatcher;
    
    this->pre_start();
    
    m_terminate = false;
    m_running = true;

    if ( m_use_dispatcher )
    {
      m_dispatcher = new Glib::Dispatcher();
      m_dispatcher->connect( sigc::mem_fun( *this, &Server::emit_proxy ) );
    }

    m_thread = Glib::Thread::create( sigc::mem_fun( *this, &Server::service_thread_main_proxy ), true );

    this->post_start();
    
    m_signal_start_stop.emit(START);
  }

  void Server::stop()
  {
    this->pre_stop();
    
    m_terminate = true;
    m_thread->join();
    m_thread = NULL;
    m_running = false;

    if ( m_dispatcher )
    {
      delete m_dispatcher;
      m_dispatcher = NULL;
    }

    this->post_stop();
    
    m_signal_start_stop.emit(STOP);
  }

  sigc::signal< void, StartStop > Server::signal_start_stop()
  {
    return m_signal_start_stop;
  }

  bool Server::is_running()
  {
    return m_running;
  }

  void Server::set_responsiveness( long r )
  {
    m_timeval.tv_usec = r;
  }

  long Server::responsiveness()
  {
    return m_timeval.tv_usec;
  }

  void Server::service_thread_main_proxy()
  {
    this->service_thread_main();
  }

  void Server::emit_proxy()
  {
    this->emit();
  }

}


