/* Host Entry
 *
 * Allows the user to enter a host name or an IP address. In addition to providing an entry widget,
 * this widget can optionally display the hostname and IP address beneath the entry widget as well as
 * an optional image that indicates whether the current entry has been found via a DNS lookup.
 *
 * As with all of the Conexus::Gtk widgets, these widgets extend the standard Gtk widgets by allowing
 * a Conexus endpoint widget to be attached. Thus, changes to the endpoint will result in changes
 * to the widget, and vice-versa. In essence, the widget becomes the 'view' on the endpoint 'model'
 * a la the model-view abstraction.
 *
 * This widget allows IPv4 and IPv6 endpoints to be attached.
 *
 * This demo provides three checkboxes to turn on/off the optional features.
 */

#include "gtkmm.h"

#include <conexus-gtkmm.h>

class Example_HostEntry : public Gtk::Window
{
  public:
    Example_HostEntry();
    virtual ~Example_HostEntry();

  protected:
    /* Member widgets */
    Conexus::Gtk::HostEntry m_HostEntry;

    typedef enum BUTTON{ HOST_LABEL, ADDRESS_LABEL, IMAGE } BUTTON;
    Gtk::CheckButton m_host_label_button;
    Gtk::CheckButton m_address_label_button;
    Gtk::CheckButton m_image_button;
    Gtk::VBox m_vbox;

    void on_button_clicked(BUTTON which);

};

/* Called by DemoWindow */
Gtk::Window* do_HostEntry()
{
  return new Example_HostEntry();
}

Example_HostEntry::Example_HostEntry()
{

  set_title("Host Entry");
  set_border_width(10);

  /* Add the HostEntry widget */
  m_vbox.pack_start( m_HostEntry );

  /* And the other widgets for this demo */
  m_vbox.pack_start( *Gtk::manage( new Gtk::HSeparator() ) );
  m_host_label_button.set_label( "Show hostname label" );
  m_address_label_button.set_label( "Show IP address label" );
  m_image_button.set_label( "Show entry validation image" );
  m_host_label_button.set_active( true );
  m_address_label_button.set_active( true );
  m_image_button.set_active( true );
  m_host_label_button.signal_toggled().connect( sigc::bind( sigc::mem_fun( *this, &Example_HostEntry::on_button_clicked ), HOST_LABEL ) );
  m_address_label_button.signal_toggled().connect( sigc::bind( sigc::mem_fun( *this, &Example_HostEntry::on_button_clicked ), ADDRESS_LABEL ) );
  m_image_button.signal_toggled().connect( sigc::bind( sigc::mem_fun( *this, &Example_HostEntry::on_button_clicked ), IMAGE ) );
  m_vbox.pack_start( m_host_label_button );
  m_vbox.pack_start( m_address_label_button );
  m_vbox.pack_start( m_image_button );

  this->add( m_vbox );

  show_all();
}

Example_HostEntry::~Example_HostEntry()
{
}

void Example_HostEntry::on_button_clicked( BUTTON which ) {
  switch ( which ) {
    case HOST_LABEL:
      m_HostEntry.set_show_hostname_label( m_host_label_button.get_active() );
      break;
    case ADDRESS_LABEL:
      m_HostEntry.set_show_address_label( m_address_label_button.get_active() );
      break;
    case IMAGE:
      m_HostEntry.set_show_host_image( m_image_button.get_active() );
      break;
  }
}

