/* Port Entry
 *
 * Allows the user to enter a port number. In addition to providing an entry widget,
 * this widget can optionally display the service name beneath the entry widget.
 *
 * As with all of the Conexus::Gtk widgets, these widgets extend the standard Gtk widgets by allowing
 * a Conexus endpoint widget to be attached. Thus, changes to the endpoint will result in changes
 * to the widget, and vice-versa. In essence, the widget becomes the 'view' on the endpoint 'model'
 * a la the model-view abstraction.
 *
 * This widget allows IPv4 and IPv6 endpoints to be attached.
 *
 * This demo provides a checkboxes to turn on/off the optional service name feature.
 */

#include "gtkmm.h"

#include <conexus-gtkmm.h>

class Example_PortEntry : public Gtk::Window
{
  public:
    Example_PortEntry();
    virtual ~Example_PortEntry();

  protected:
    /* Member widgets */
    Conexus::Gtk::PortEntry m_PortEntry;

    Gtk::CheckButton m_servicename_button;
    Gtk::VBox m_vbox;

    void on_button_clicked( );

};

/* Called by DemoWindow */
Gtk::Window* do_PortEntry()
{
  return new Example_PortEntry();
}

Example_PortEntry::Example_PortEntry()
{

  set_title("Port Entry");
  set_border_width(10);

  /* Add the port entry widget */
  m_vbox.pack_start( m_PortEntry );

  /* Now add the other widgets for this demo */
  m_vbox.pack_start( *Gtk::manage( new Gtk::HSeparator() ) );
  m_servicename_button.set_label( "Show service name label" );
  m_servicename_button.set_active( true );
  m_servicename_button.signal_toggled().connect( sigc::mem_fun( *this, &Example_PortEntry::on_button_clicked ) );
  m_vbox.pack_start( m_servicename_button );

  this->add( m_vbox );

  show_all();
}

Example_PortEntry::~Example_PortEntry()
{
}

void Example_PortEntry::on_button_clicked( ) {
  m_PortEntry.set_show_servicename_label( m_servicename_button.get_active() );
}

