/* Spinner
 *
 * Spins a papyrus canvas based on data flow.
 *
 */

#include "gtkmm.h"

#include <cstdlib>

#include <conexus-gtkmm.h>
#include <conexus.h>

class Example_Spinner : public Gtk::Window
{
  public:
    Example_Spinner();
    virtual ~Example_Spinner();

  protected:
    /* Member widgets */
    Conexus::Gtk::Spinner m_Spinner;

    Conexus::IPv4::UDP::pointer m_sender, m_receiver;

    Gtk::CheckButton m_startstop;
    Gtk::VBox m_vbox;

    Glib::RefPtr<Glib::TimeoutSource> m_timeout;

    void on_start_stop_toggled();

    char* m_data;

    bool on_timeout();

};

/* Called by DemoWindow */
Gtk::Window* do_Spinner()
{
  return new Example_Spinner();
}

Example_Spinner::Example_Spinner()
{

  m_receiver = Conexus::IPv4::UDP::create();
  m_sender = Conexus::IPv4::UDP::create();

  set_title("Spinner");
  set_border_width(10);

  m_startstop.set_label("Run");

  m_data = new char[2048];
  memset( m_data, 0x00, 2048 );

  m_Spinner.add_endpoint(m_receiver);

  m_receiver->local_address().set_port(3500);
  m_receiver->start();

  m_sender->remote_address().set_address("localhost");
  m_sender->remote_address().set_port(3500);
  m_sender->connect();

  m_vbox.pack_start( m_Spinner );
  m_vbox.pack_start( *Gtk::manage( new Gtk::HSeparator() ), Gtk::PACK_EXPAND_WIDGET, 10 );
  m_vbox.pack_start( m_startstop );

  m_startstop.signal_toggled().connect( sigc::mem_fun(*this, &Example_Spinner::on_start_stop_toggled));

  this->add( m_vbox );

  show_all();
}

Example_Spinner::~Example_Spinner()
{
  delete[] m_data;
}

void Example_Spinner::on_start_stop_toggled()
{
  if ( m_startstop.get_active() )
  {
    m_timeout = Glib::TimeoutSource::create(50);
    m_timeout->connect( sigc::mem_fun( *this, &Example_Spinner::on_timeout ) );
    m_timeout->attach( Glib::MainContext::get_default() );
  }
  else
  {
    m_timeout->destroy();
    m_timeout.clear();
  }
}

bool Example_Spinner::on_timeout()
{
  int size = rand() % 2048;
  m_sender->write( m_data, size );
  return true;
}

